/*
FakeBO - fakes Back Orifice Server and logs every attempt to file/stdout
Code Maintainer: Vlatko Kosturjak, Kost <kost@iname.com>
Distributed Under GNU Public License.
*/

#define FAKEBOVER "0.1.6"
#define FAKEBOCONF "/etc/fakebo.conf"

#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <limits.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <signal.h>

#define TRUE 1
#define FALSE 0
#define ARGSIZE 256
#define BUFFSIZE 1024
#define LINE_SIZE 1024	/* in the config file */
#define MAGICSTRING "*!*QWTY?"
#define MAGICSTRINGLEN 8

#define MAXNUMCOMMANDS 64

/* Configuration parameters and their default values */
#define NB_PARAMS 12
char porttolisten[10] = "31337";
char msgtoanswer[512] = "Didn't you know that LINUX cannot be "
	"(kvazi)hacked with BO?";
char logfile[255] = "stdout";
int logconnection = 1;
int logreceivedpackets = 1;
int logsendingpackets = 1;
int lognotbopackets = 1;
int sendfakereply = 1;
char machinename[20] = "DEFAULT";
int logtimeanddate = 1;
int silentmode = 0;
int bufferedlogging = 0;

/* This is for parsing the config file and printing debug information */
char *keywords[NB_PARAMS] = {
	"port", "message", "logfile", "logconnection",
	"logreceivedpackets", "logsendingpackets",
	"lognotbopackets", "sendfakereply", "machinename",
	"logtimeanddate", "silentmode", "bufferedlogging"
};
void *addresses[NB_PARAMS] = {
	porttolisten, msgtoanswer, logfile, &logconnection,
	&logreceivedpackets, &logsendingpackets,
	&lognotbopackets, &sendfakereply, machinename,
	&logtimeanddate, &silentmode, &bufferedlogging
};
char *in_formats[NB_PARAMS] = {
	"%s", "\"%[^\"]\"", "\"%[^\"]\"", "%d",
	"%d", "%d",
	"%d", "%d", "%s",
	"%d", "%d", "%d"
};
char *out_formats[NB_PARAMS] = {
	"%s", "\"%s\"", "%s", "%d",
	"%d", "%d",
	"%d", "%d", "%s",
	"%d", "%d", "%d"
};

/* Help the user */
char usage[] =
"Usage: %s [ -c config_file ] [ -d ] [ -h ]\n"
"	-c	specify the config file. Default is "FAKEBOCONF"\n"
"	-d	print debugging information\n"
"	-h	print this help message\n";

typedef unsigned char byte;

typedef struct
{
        byte magic[8] __attribute__((packed));  // signature  (8 bytes)
        unsigned long len __attribute__((packed));      // lenght of entire packet(32bit integer)
        long id __attribute__((packed));                // id of packet (32bit integer)
        byte t __attribute__((packed));                 // type of operation (2 most of frag,6 op)
} boheader;


typedef union
{
	boheader hdr;
	char buf[BUFFSIZE];
} bopacket;

char bocommands[MAXNUMCOMMANDS][50]=
{
"Error type","Ping Packet","System Reboot","System Lock Up",
"List System Passwords","View Console","Get System Information",
"Log Pressed Keys","Send KeyPress Log","Show A Dialog Box",
"Delete An Value from The Registry","Create TCP redirection(proxy)",
"Delete TCP redirection","List TCP redirections",
"Start Application","End Application","Export a share resource","Cancel share export",
"Show Export List","Resend Packet","Enable HTTP Server","Disable HTTP Server",
"Resolve Host Name","Compress a File","Uncompress a File","Plug-in execute",
"0x1a???","0x1b???","0x1c???","0x1d???","0x1e???","0x1f???",
"Show active processes","Kill a process","Start a process","Create a key in the registry",
"Set the Value of a key in registry","Delete a key in registry",
"Enumerate registry keys","Enumerate registry values",
"Capture a static image","Capture a video stream","Play a sound file","Show Available Video capture devices",
"Capture the screen to a file","Start sending a file using TCP",
"Start recieving a file using TCP","List (running) plug-ins",
"Kill Plugin","List directory","0x32???","0x33???","Find a file","Delete a file",
"View file contents","Rename a file","Copy a file","List all network devices",
"Connect to network resource","End connection of a network resource",
"Show NetWork Connections","Create Directory (folder)","Remove directory","Show Running Applications"
};
static char notyetimplemented[]="Type Of Packet Unknown";

static long holdrand = 1L;
char g_password[ARGSIZE];

static volatile int stop = FALSE;   /* for signal handling */

FILE *fptolog;

static void sighandler(int sig)
{
        static char *sigs[] =
        {
                "EXIT", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT", "BUS", "FPE",
                "KILL", "USR1", "SEGV", "USR2", "PIPE", "ALRM", "TERM", "STKFLT",
                "CHLD", "CONT", "STOP", "TSTP", "TTIN", "TTOU", "URG", "XCPU",
                "XFSZ", "VTALRM", "PROF", "WINCH", "IO", "PWR", "UNUSED", "ZERR",
                "DEBUG", (char *)(NULL)
        };
        int i;

        if (sig < 0)
                sig = -sig;
        for (i = 0; sigs[i] != (char *)(NULL); i++) ;
        sig %= i;
        fprintf(stderr, "Caught SIG%s, shutting down the server...\n", sigs[sig]);
        stop = TRUE;
}


static void setsignal(int sig, void (*sighnd)(int signo))
{
#ifndef NeXT
        struct sigaction sa;
#endif

#ifdef NeXT
        signal(sig, sighnd);
#else
        sa.sa_handler = sighnd;
        sigemptyset(&(sa.sa_mask));
        sa.sa_flags = 0;
        sigaction(sig, &(sa), (struct sigaction *)(NULL));
#endif
}

static void siginit(void)
{
        stop = FALSE;
        setbuf(stdin, (char *)(NULL));
        setbuf(stdout, (char *)(NULL));
        setbuf(stderr, (char *)(NULL));
        setsignal(SIGPIPE, SIG_IGN);
        setsignal(SIGINT, sighandler);
        setsignal(SIGQUIT, sighandler);
        setsignal(SIGTERM, sighandler);
        setsignal(SIGBUS, sighandler);
        setsignal(SIGFPE, sighandler);
        setsignal(SIGILL, sighandler);
        setsignal(SIGHUP, sighandler);
        setsignal(SIGABRT, sighandler);
        setsignal(SIGSEGV, sighandler);
        setsignal(SIGCHLD, sighandler);
        setsignal(SIGKILL, sighandler);
        
}

void whatistime (char *timenow)
{
	struct tm *curtime;
	time_t cartime;
	
	time (&cartime);
	curtime=localtime (&cartime);
	sprintf (timenow, "%d-%d-%d %d:%d", curtime->tm_year, curtime->tm_mon, 
	curtime->tm_mday, curtime->tm_hour, curtime->tm_min);	
}
	
void bocommandinit (void)
{
	byte i;
	char tmpstr[80];
	
	for (i=0;i<MAXNUMCOMMANDS;i++)
	{
		strcpy (tmpstr,bocommands[i]);
		sprintf (bocommands[i],"0x%02X %s",i,tmpstr);
	}
}

void loadconfig (char *config_file)
{
	FILE *fp;
	char line[LINE_SIZE], *pos;
	int var, linenb;

	if (config_file != NULL) fp = fopen(config_file, "rt");
	else fp=fopen (FAKEBOCONF,"rt");
	if (fp==NULL)
		fprintf (stderr, "Could not open config file, using defaults.\n");
	else
	{
		linenb = 0;
		while (fgets(line, LINE_SIZE, fp) != NULL) {
			linenb++;
			if ((line[0] == '#') || (line[0] == '\n')) continue; /* comment */
			for (var=0 ; var<NB_PARAMS ; var++) {
				if (strstr(line, keywords[var])!=line) continue;
				pos = line + strlen(keywords[var]);
				/* keyword must be followed by an espace */
				if (*(pos++) != ' ') continue;
				sscanf(pos, in_formats[var], addresses[var]);
				break;
			}
			if (var == NB_PARAMS)
				fprintf(stderr, "unrecognized command in config file"
					" %s, line %d:\n%s\n", config_file, linenb, line);
		}
		fclose (fp);
	}
	if (strcmp (logfile,"stdout")==0)
	{
		printf ("Logging to stdout.\n");
		fptolog=stdout;
	}
	else
	{
		fptolog=fopen (logfile,"at");
		if (fptolog==NULL)
		{
			printf ("Error opening logfile (%s), using stdout\n",logfile);
			fptolog=stdout;
		}
		else
		{
			printf ("Using logfile %s\n",logfile);
		}
	}
	if (bufferedlogging==0) setbuf (fptolog,(char *)NULL);
}

void debugconfig (void)
{
	int var;

	fprintf (stderr, "---[Config Parametars]---\n");
	for (var=0 ; var<NB_PARAMS; var++) {
		fprintf(stderr, "%s ", keywords[var]);
		if (strcmp(out_formats[var], "%d") == 0)
			fprintf(stderr, out_formats[var], *(int *)addresses[var]);
		else
			fprintf(stderr, out_formats[var], addresses[var]);
		fprintf(stderr, "\n");
	}
}


char *typeofbocommand (byte t)
{
	byte i;
	
	if (t & 0x80) t= t - 0x80;
	if (t & 0x40) t= t - 0x40;
	for (i=0; i<MAXNUMCOMMANDS; i++)
	if ((t & i)==t) 
	{
		return (bocommands[i]);
	}
	return (notyetimplemented);
}


void debugspacket (bopacket *vamo)
{
	printf ("MAGIC=%.8s\t",vamo->hdr.magic);
	printf ("Len=%ld\t",vamo->hdr.len);
	printf ("ID=%ld\t",vamo->hdr.id);
	printf ("Type=0x%02X\n",vamo->hdr.t);
	printf ("%s\n",typeofbocommand(vamo->hdr.t));
	printf ("Data\n");
	printf ("%s\n",&vamo->buf[sizeof(vamo->hdr)]);
	printf ("End.\n");
}

void debugcpacket (bopacket *vamo)
{
	char *c;

    printf ("MAGIC=%.8s\t",vamo->hdr.magic);
    printf ("Len=%ld\t",vamo->hdr.len);
    printf ("ID=%ld\t",vamo->hdr.id);
    printf ("Type=0x%02X\n",vamo->hdr.t);
    printf ("%s\n",typeofbocommand(vamo->hdr.t));
    printf ("Data1\n");
    c=&vamo->buf[sizeof(vamo->hdr)];
    printf ("%s\n",c);
    printf ("Data2\n");
    c+=(strlen(c)+1);
	printf ("%s\n",c);
	printf ("End.\n");
}

void logcpacket (bopacket *vamo)
{
        char *c;

	switch (logreceivedpackets)
	{
		case 3: 
		    fprintf (fptolog, "\tMagic=%.8s\t",vamo->hdr.magic);
		    fprintf (fptolog, "Len=%ld\t",vamo->hdr.len);
		    fprintf (fptolog, "ID=%ld\t",vamo->hdr.id);
		    fprintf (fptolog, "Type=0x%02X\n",vamo->hdr.t);			
		case 2:
	  	    c=&vamo->buf[sizeof(vamo->hdr)];
		    fprintf (fptolog, "\t(%s)",c);
		    c+=(strlen(c)+1);
		    fprintf (fptolog, "(%s)\n",c);			
		case 1:
			fprintf (fptolog,"\tCommand: %s\n",typeofbocommand(vamo->hdr.t));
		default: 
			break;
	}
}

void logspacket (bopacket *vamo)
{
	switch (logsendingpackets)
	{
		case 3:
        		fprintf (fptolog, "\tMagic=%.8s\t",vamo->hdr.magic);
        		fprintf (fptolog, "Len=%ld\t",vamo->hdr.len);
       			fprintf (fptolog, "ID=%ld\t",vamo->hdr.id);
        		fprintf (fptolog, "Type=0x%02X\n",vamo->hdr.t);
		case 2:
        		fprintf (fptolog, "\t(%s)\n",&vamo->buf[sizeof(vamo->hdr)]);
		case 1:
                        fprintf (fptolog, "\tCommand: %s\n",typeofbocommand(vamo->hdr.t));
		default:
			break;
	}
}
                                                                                                          
void  msrand (unsigned int seed )
{
	holdrand = (long)seed;
}
  
int mrand ( void)
{
    return(((holdrand = holdrand * 214013L + 2531011L) >> 16) & 0x7fff);
}


unsigned int getkey()
{
  int x, y;
  unsigned int z;

  y = strlen(g_password);
  if (!y)
    return 31337;
  else {
    z = 0;
    for (x = 0; x < y; x++)
      z+= g_password[x];

    for (x = 0; x < y; x++)
      {
    if (x%2)
      z-= g_password[x] * (y-x+1);
    else
      z+= g_password[x] * (y-x+1);
    z = z%RAND_MAX;
      }
    z = (z * y)%RAND_MAX;
    return z;
  }
}

void cryptpacket(unsigned char *buff, int len)
{
  int y;

  if (!len)
    return;

  msrand(getkey());
  for (y = 0; y < len; y++)
    buff[y] = buff[y] ^ (mrand()%256);
}
    

int atoport(char *service, char *proto)
{
  int port;
  long int lport;
  struct servent *serv;
  char *errpos;

  serv = getservbyname(service, proto);
  if (serv != NULL)
    port = serv->s_port;
  else { 
    lport = strtol(service,&errpos,0);
    if ( (errpos[0] != 0) || (lport < 1) || (lport > 65535) )
      return -1; 
    port = htons(lport);
  }
  return port;
}

void intro (void)
{
	printf ("FakeBo %s. Coded by KoSt and DryLLaR. Distributed under GNU public license.\n\n",FAKEBOVER);
}

int main(int argc, char *argv[])
{
	bopacket bo;
	unsigned long slen=0;
	int opt;
	char *config_file = NULL;
	int debug = FALSE;
  int sock;  
  struct sockaddr_in client, server; 
  int recvd;
  int structlength; 
  int port;
	struct linger linger;
	struct timeval tv;
	struct hostent *he;
	fd_set fds;
	char *msg;
    char ad[256],tmpstr[256], *ipaddr;
    int tosendpacket=1;

	intro();
	while ((opt=getopt(argc, argv, "c:dh")) != EOF) switch (opt) {
		case 'c' : config_file = optarg; break;
		case 'd' : debug = TRUE; break;
		case 'h' : printf(usage, argv[0]); return 0;
		default : fprintf(stderr, usage, argv[0]); return -1;
	}
	loadconfig(config_file);
	if (debug) debugconfig();

  port = atoport(porttolisten,"udp");

  memset (g_password, 0, sizeof(g_password));

  if (port < 0) {
    fprintf(stderr,"Unable to find port %s.\n",argv[1]);
    exit(EXIT_FAILURE);
  }

  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if (sock < 0) {
    perror("socket");
    exit(EXIT_FAILURE);
  }

	siginit();  /* initialize signal handlers */
	if (argc>1)	bocommandinit(); 
	linger.l_onoff = 0;          /* dont linger */
	setsockopt(sock, SOL_SOCKET, SO_LINGER, (void *)&linger, sizeof(linger) );
    
  memset((char *) &client, 0, sizeof(client));
  client.sin_family = AF_INET;
  client.sin_addr.s_addr = htonl(INADDR_ANY);
  client.sin_port = port;

  memset((char *) &server, 0, sizeof(server));
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = htonl(INADDR_ANY);
  server.sin_port = port;

  structlength = sizeof(server);
  if (bind(sock, (struct sockaddr *) &server, structlength) < 0) {
    perror("bind");
    exit(EXIT_FAILURE);
  }

	printf ("Fake BO Server Booted Up. Waiting for connections...(CTRL+C for end)\n");
  while (stop==FALSE)
  {
    slen=0;
	memset ((char *)&bo,0,sizeof(bo));
    structlength = sizeof(client);
	recvd=0;
	recvd = recvfrom(sock, &bo, sizeof(bo), 0, (struct sockaddr *)&client, &structlength);
	if (stop==TRUE) continue;
    if (recvd < 0)
    {
      perror("recvfrom");
      exit(EXIT_FAILURE);
    }
	cryptpacket ((unsigned char *)&bo, recvd);
	slen=bo.hdr.len;
    
    he = gethostbyaddr((char *)(&(client.sin_addr)),
    sizeof(client.sin_addr), AF_INET);
    ipaddr = inet_ntoa(client.sin_addr);
    if (he != (struct hostent *)(NULL))
         sprintf(ad, "%s (%s)", he->h_name, ipaddr);
    else
         sprintf(ad, "%s", ipaddr);
	
	whatistime (tmpstr);
	if (logtimeanddate!=0) fprintf (fptolog, "%s ",tmpstr);
	if (logconnection!=0) fprintf (fptolog, "Packet from %s:",ad);
	
    if ((strncmp(bo.hdr.magic,MAGICSTRING,MAGICSTRINGLEN)==0) && (slen!=0))
    {
	if (logconnection!=0) fprintf (fptolog, "BO PACKET!\n");

	/* want to turn off debugging of packets??? */
	logcpacket (&bo);

    FD_ZERO(&fds);
    FD_SET(sock, &fds);
    tv.tv_sec = 10;
    tv.tv_usec= 0;
    if (select(sock+1, NULL, &fds, NULL, &tv) == 0)
    {
        perror ("select (timeout error)");
        continue;
    }
	msg = &bo.buf[sizeof(bo.hdr)];
	if ((sendfakereply==1) && (bo.hdr.t == 0x01)) 
	{
		strcpy (msg,"  !PONG!1.20!");
		strcat (msg,machinename);
		strcat (msg,"!");
	}
	else strcpy (msg, msgtoanswer);	

	tosendpacket=0;
	if (silentmode==0) tosendpacket=1;
	else if ((bo.hdr.t == 0x01) && (sendfakereply==1)) tosendpacket=1;
	
		bo.hdr.len += strlen(msg)+2;
		slen=bo.hdr.len;
		holdrand = 1L;

      if (tosendpacket)
      {
        if (logconnection!=0) fprintf (fptolog, "Sending packet to %s...\n",ad);
        logspacket(&bo);

        cryptpacket ((unsigned char *)&bo, sizeof(bo));      
      	if (FD_ISSET(sock,&fds)) 
      	if (sendto(sock, &bo, slen, 0, (struct sockaddr *) &client, sizeof(client)) != slen)
      	{
        	perror("sendto:");
      	}
      }
      
    }
    else 
    {
	if (logconnection!=0) fprintf (fptolog, "Non-BO Packet\n");
	if (lognotbopackets!=0)
	{
		fprintf (fptolog,"\t%s\n",bo.buf);
	}
    }
  }
  if (fptolog!=stdout) fclose (fptolog);
	return (0);
}

